import Axios, { AxiosInstance, AxiosRequestConfig, AxiosResponse } from 'axios';
import { ElLoading, ElMessage } from 'element-plus';
import qs from 'qs';
import uuidv1 from './uuid';

interface CustomAxiosRequestConfig extends AxiosRequestConfig {
  isShowLoading?: boolean;
}
// 定义出参类型
interface OwnResponse<T> {
  result: T;
  resultMessage: string;
  resultCode: string;
  success: boolean;
  [propName: string]: any;
}
const defaultConfig: AxiosRequestConfig = {
  // eslint-disable-next-line no-nested-ternary
  baseURL: import.meta.env.MODE === 'mock' ? '/api/mock/72' : import.meta.env.MODE === 'localDev' ? '/api' : '',
  headers: {
    'Content-Type': 'application/json',
    Authorization: `Bearer ${localStorage.getItem('token')}`
  },
  validateStatus(status: number) {
    return status >= 200 && status < 400;
  }
};
const LOAD_DEFAULT = true;
class Request {
  public instance: AxiosInstance;

  public loadingInstance: any;

  public isShowLoading = LOAD_DEFAULT;

  constructor(config: CustomAxiosRequestConfig) {
    this.instance = Axios.create(config);
    this.instance.interceptors.request.use((requestConfig) => {
      requestConfig.headers['x-b3-traceId'] = uuidv1();
      requestConfig.headers['x-b3-spanId'] = uuidv1();
      if (this.isShowLoading) {
        this.loadingInstance = ElLoading.service({
          fullscreen: true,
          text: '加载中',
          background: 'rgba(0, 0, 0, 0.5)'
        });
      }
      return requestConfig;
    });
    this.instance.interceptors.response.use(
      (response) => {
        this.loadingInstance?.close();

        return response;
      },
      (err) => {
        this.loadingInstance?.close();
        return Promise.reject(err);
      }
    );
  }

  downloadFile = function (url: any, fileName: any) {
    const token = `Bearer ${localStorage.getItem('token')}`;
    Axios.defaults.headers.common.Authorization = token;
    Axios({
      method: 'post',
      responseType: 'blob',
      url,
      data: qs.stringify({ fileName })
    }).then((res) => {
      if (res.status === 200) {
        const aLink = document.createElement('a');
        const blob = new Blob([res.data], { type: 'text/plain;charset=utf-8' });
        aLink.href = URL.createObjectURL(blob);
        aLink.setAttribute('download', fileName); //  设置下载文件名称
        aLink.click();
        URL.revokeObjectURL(aLink.href);
        document.body.appendChild(aLink);
      } else {
        ElMessage.error('系统异常，下载失败');
      }
    });
  };

  request<T>(config: CustomAxiosRequestConfig): Promise<T> {
    if (config.isShowLoading === false) {
      this.isShowLoading = false;
    }
    return new Promise<T>((resolve, reject) => {
      this.instance
        .request(config)
        .then((res: AxiosResponse<OwnResponse<T>>) => {
          this.isShowLoading = LOAD_DEFAULT;
          if (res.data) {
            if (res.data.success) {
              resolve(res.data.result);
            } else {
              res.data.error = true;
              resolve(res.data as any);
              ElMessage.error(res.data.resultMessage);
            }
          }
        })
        .catch((err) => {
          this.isShowLoading = LOAD_DEFAULT;
          ElMessage.error('接口异常');
          reject(err);
        });
    });
  }

  // 约束入参出参类型
  get<T, K>(url: string, data?: T, config?: CustomAxiosRequestConfig): Promise<K> {
    return this.request<K>({ ...config, url, params: data, method: 'GET' });
  }

  post<T, K = any>(url: string, data?: T, config?: CustomAxiosRequestConfig): Promise<K> {
    return this.request<K>({ ...config, data, url, method: 'POST' });
  }
}

export default new Request(defaultConfig);
